#include "sdkconfig.h"
#include "soc/rtc_cntl_reg.h"
#include "soc/rtc_io_reg.h"
#include "soc/soc_ulp.h"
#include "soc/sens_reg.h"

#include "macro.S"

	.bss

	.global stack
stack:
	.skip 24

	.global stackEnd
stackEnd:
	.long 0

	.global dataR0
dataR0:
	.long 0

	.global scratchpad_memory
scratchpad_memory:	// Return 9 Bytes of data after an 0xBE command.
	.long 0		// Byte 0: Temperature LSB
	.long 0		// Byte 1: Temperature MSB
	.long 0		// Byte 2: Th Register
	.long 0		// Byte 3: Tl Register
	.long 0 	// Byte 4: Configuraion Register
	.long 0		// Byte 5: Reserved
	.long 0		// Byte 6: Reserved
	.long 0		// Byte 7: Reserved
	.long 0		// Byte 8: CRC value

.data

   .global set_out_one_wire_pin
set_out_one_wire_pin:
	WRITE_RTC_REG(RTC_GPIO_ENABLE_W1TS_REG, RTC_GPIO_ENABLE_W1TS_S + 0, 1, 1)
	ret

   .global set_in_one_wire_pin
set_in_one_wire_pin:
	WRITE_RTC_REG(RTC_GPIO_ENABLE_W1TC_REG, RTC_GPIO_ENABLE_W1TC_S + 0, 1, 1)
	ret

   .global set_one_wire_pin
set_one_wire_pin:
	WRITE_RTC_REG(RTC_GPIO_OUT_W1TS_REG, RTC_GPIO_ENABLE_W1TS_S + 0, 1, 1)
	ret

   .global clear_one_wire_pin
clear_one_wire_pin:
	WRITE_RTC_REG(RTC_GPIO_OUT_W1TC_REG, RTC_GPIO_ENABLE_W1TC_S + 0, 1, 1)
	ret

	.text
	.global entry
entry:
	move r3, stackEnd

	psr
	jump get_temp

	move r1, scratchpad_memory
	add r1, r1, 4
	ld r0, r1, 0
	move r1, dataR0
	st r0, r1, 0

	jump exit_loop

rst_pulse:
	GPIO_OUT
	GPIO_LOW
	wait(4350) //500 us
	GPIO_HIGH
	GPIO_IN
	wait(240) //30us
	.global get_pin0
get_pin0:
	READ_RTC_REG(RTC_GPIO_IN_REG, RTC_GPIO_IN_NEXT_S + 0, 1) // r0 holds input result
	wait(3760) //470us
	ret

get_temp:
	psr
	jump rst_pulse
	//send 0xCC and 0x44 command
	move r2, 0xCC
	psr
	jump send_byte
	move r2, 0x44
	psr
	jump send_byte
	//delay 750ms, looping 750x in delay_ms
	//0x44 convert command: convertion is slow, 750 milliseconds
	//delay(4 sec)
	move r0, 750
	psr
	jump delay_ms
	psr
	jump rst_pulse
	//sent 0xCC and 0xBE command
	move r2, 0xCC
	psr
	jump send_byte
	move r2, 0xBE //read scratchpad(9 bytes, last byte is CRC)
	psr
	jump send_byte
	//read 9 bytes from scratchpad memory
	stage_rst
	move r0, scratchpad_memory
get_temp_9bytes:
	push r0
	psr
	jump read_byte
	pop r0
	st r2, r0, 0
	add r0, r0, 1
	stage_inc 1
	jumps get_temp_9bytes, 9, LT

	psr
	jump rst_pulse

	psr
	jump checkCRC8_bytes

	move r1, crc8_check
	st r2, r1, 0

	ret

read_byte:
	move r2, 0
	move r0, 0
read_byte_loop:
	push r0
	psr
	jump read_bit
	move r1, r0
	pop r0
	lsh r1, r1, r0
	or r2, r2, r1
	wait(120) //15 us
	add r0, r0, 1
	jumpr read_byte_loop, 8, LT
	ret

//R0: (1bit) stores GPIO input on return.
// since R2 is used to store all the bits
read_bit:
	GPIO_OUT
	GPIO_LOW
	wait(10) //2 us
	GPIO_HIGH
	wait(100) //15 us
	GPIO_IN
	.global get_pin1
get_pin1:
	READ_RTC_REG(RTC_GPIO_IN_REG, RTC_GPIO_IN_NEXT_S + 0, 1)
	ret

send_byte:
	move r1, r2
	move r0, 0
send_byte_loop:
	rsh r2, r1, r0
	and r2, r2, 0x01
	push r0
	push r1
	psr
	jump send_bit
	pop r1
	pop r0
	add r0, r0, 1
	jumpr send_byte_loop, 8, LT
	wait(800) //100us
	ret

send_bit:
	GPIO_OUT
	GPIO_LOW
	wait(30) //5us
	sub r2, r2, 0
	jump send_bit_low, eq
send_bit_high:
	GPIO_HIGH
send_bit_low:
	wait(640) /* 80us */
	GPIO_HIGH
	ret

exit_loop:
	//delay(4 sec)
	move r0, 4000
	psr
	jump delay_ms

	jump wake_up
	/* End program */

	//delay
delay_ms:
	delay_ms_loop:
	sub r0, r0, 1
	wait(7990) /* 1millsecond = 1000 microsecond */
	jumpr delay_ms_loop, 1, GE
	ret
